(function () {
	function loadScriptList(scriptList) {
		var oHead = document.head || document.getElementsByTagName('head')[0];

		var randomizer = Math.random().toString(10).substring(2, 8);

		return new Promise(function (resolve, reject) {

			setTimeout(function () {
				var error = new Error();
				error.Code = ErrorCodes.StaptupScriptTimeout;
				reject(error);
			}, 20000);

			var onAfterLoadSingle = _.after(scriptList.length, resolve);
			_.each(scriptList, loadSingleScript);

			function loadSingleScript(script) {
				var sc = document.createElement('script');

				sc.onload = onAfterLoadSingle;
				sc.onerror = function () {
					var error = new Error('error load: ' + script);
					error.Code = ErrorCodes.StartupScriptLoad;
					reject(error);
				};

				sc.setAttribute('type', 'text/javascript');
				sc.setAttribute('src', script + '?rnd=' + randomizer);

				oHead.appendChild(sc);
			}
		});
	}

	/**
	 * Для iOS 14.5 и выше делаем запрос на отслеживание пользователя перед показом рекламы
	 */
	function patchAdmobShow() {
		window.application.admobShow = function() {
			admob.getTrackingStatus().then(function (status) {
				if (status === 'notDetermined') {
					admob.trackingStatusForm().then(function () { admob.rewardvideo.show(); });

				} else {
					admob.rewardvideo.show();
				}
			});
		}
	}

	/**
	 * Начиная с 6.0 версии cordova-plugin-appsflyer-sdk trackEvent заменён на logEvent
	 */
	function patchAppsFlyer() {
		if (window.plugins.appsFlyer) {
			if (!window.plugins.appsFlyer.trackEvent) {
				window.plugins.appsFlyer.trackEvent = window.plugins.appsFlyer.logEvent;
			}

			if (!window.plugins.appsFlyer.enableUninstallTracking) {
				window.plugins.appsFlyer.enableUninstallTracking = function (deviceToken, successCB, errorCB) {
					window.plugins.appsFlyer.registerUninstall(deviceToken);
				}
			}
		}
	}

	function loadFrame() {
		if (!window.application) {
			var error = new Error();
			error.Code = ErrorCodes.NoApplication;
			throw error;
		} else {
			patchAdmobShow();

			application.run(4); // version ?
		}
	}

	function onDeviceReady() {
		// если соединения нет никакого, то об этом говорим сразу
		if (navigator.connection && navigator.connection.type === 'none') {
			showConnectionErrorMessage(ErrorCodes.NoConnection);
			location.reload();
			return;
		}
		// делаем подмену методов старого метода trackEvent на logEvent
		patchAppsFlyer();

		// праячем status bar
		AndroidFullScreen.immersiveMode(function () {
			console.log('Immersive mode enabled');
		}, function (err) {
			console.log(err);
		});

		loadScriptList(scriptList)
			.then(loadFrame)
			.catch(function (error) {
				if (error) {
					if (error.Code) {
						showConnectionErrorMessage(error.Code);
					}
				}

				location.reload();
			});
	}

	function showConnectionErrorMessage(step) {
		navigator.globalization.getLocaleName(function (data) {
			var locale = data.value.split('-')[0];
			showMessageForLocale(step, locale);
		}, function () {
			showMessageForLocale(step);
		});

		function showMessageForLocale(step, locale) {
			if (!connectionErrorText[locale]) {
				locale = 'en';
			}

			var msg = _.template(connectionErrorText[locale])({errorCode: step});
			alert(msg);
		}
	}

	var connectionErrorText = {
		ru: 'Невозможно запустить приложение, проверьте подключение к интернету! (код ошибки: <%= errorCode %>)',
		en: 'Unable to start the application, check the the Internet connection! (error code: <%= errorCode %>)',
		jp: 'アプリケーションを起動することができません、インターネット接続を確認！'
	};

	var ErrorCodes = {
		NoConnection: 10,
		StaptupScriptTimeout: 11,
		StartupScriptLoad: 12,
		NoApplication: 13
	};

	document.addEventListener('deviceready', onDeviceReady, false);
	window.fabric = {
		Crashlytics: {
			addLog: _.noop,
			setStringValueForKey: _.noop,
			sendNonFatalCrash: _.noop
		},
		Answers: {
			sendCustomEvent: _.noop,
			sendLogIn: _.noop,
			sendPurchase: _.noop,
		}
	}
})();
